@@ -3,7 +3,6 @@ package config
 import (
 	"fmt"
 	"io/ioutil"
-	"net"
 	"os"
 	"path/filepath"
 	"strings"
@@ -135,14 +134,14 @@ func writeTestConfigFile(path string, overrides ...string) error {
 	const defaultData string = `[default]
 mtu=1500
 conntrack-zone=64321
-cluster-subnets=10.129.0.0/14/23
+cluster-subnets=10.132.0.0/14/23
 
 [kubernetes]
 kubeconfig=/path/to/kubeconfig
 apiserver=https://1.2.3.4:6443
 token=TG9yZW0gaXBzdW0gZ
 cacert=/path/to/kubeca.crt
-service-cidr=172.18.0.0/24
+service-cidrs=172.18.0.0/24
 no-hostsubnet-nodes=label=another-test-label
 
 [logging]
@@ -154,13 +153,13 @@ conf-dir=/etc/cni/net.d22
 plugin=ovn-k8s-cni-overlay22
 
 [ovnnorth]
-address=ssl://1.2.3.4:6641
+address=ssl:1.2.3.4:6641
 client-privkey=/path/to/nb-client-private.key
 client-cert=/path/to/nb-client.crt
 client-cacert=/path/to/nb-client-ca.crt
 
 [ovnsouth]
-address=ssl://1.2.3.4:6642
+address=ssl:1.2.3.4:6642
 client-privkey=/path/to/sb-client-private.key
 client-cert=/path/to/sb-client.crt
 client-cacert=/path/to/sb-client-ca.crt
@@ -171,6 +170,10 @@ interface=eth1
 next-hop=1.3.4.5
 vlan-id=10
 nodeport=false
+
+[hybridoverlay]
+enabled=true
+cluster-subnets=11.132.0.0/14/23
 `
 
 	var newData string
@@ -189,14 +192,6 @@ nodeport=false
 	return ioutil.WriteFile(path, []byte(newData), 0644)
 }
 
-func mustParseCIDR(cidr string) *net.IPNet {
-	_, net, err := net.ParseCIDR(cidr)
-	if err != nil {
-		panic("bad CIDR string constant " + cidr)
-	}
-	return net
-}
-
 var _ = Describe("Config Operations", func() {
 	var app *cli.App
 	var cfgFile *os.File
@@ -210,7 +205,7 @@ var _ = Describe("Config Operations", func() {
 
 	BeforeEach(func() {
 		// Restore global default values before each testcase
-		RestoreDefaultConfig()
+		PrepareTestConfig()
 
 		app = cli.NewApp()
 		app.Name = "test"
@@ -226,12 +221,6 @@ var _ = Describe("Config Operations", func() {
 	})
 
 	It("uses expected defaults", func() {
-		// Don't pick up defaults from the environment
-		os.Unsetenv("KUBECONFIG")
-		os.Unsetenv("K8S_CACERT")
-		os.Unsetenv("K8S_APISERVER")
-		os.Unsetenv("K8S_TOKEN")
-
 		app.Action = func(ctx *cli.Context) error {
 			cfgPath, err := InitConfigSa(ctx, kexec.New(), tmpDir, nil)
 			Expect(err).NotTo(HaveOccurred())
@@ -247,12 +236,14 @@ var _ = Describe("Config Operations", func() {
 			Expect(Kubernetes.CACert).To(Equal(""))
 			Expect(Kubernetes.Token).To(Equal(""))
 			Expect(Kubernetes.APIServer).To(Equal(DefaultAPIServer))
-			Expect(Kubernetes.ServiceCIDR).To(Equal("172.16.1.0/24"))
+			Expect(Kubernetes.RawServiceCIDRs).To(Equal("172.16.1.0/24"))
 			Expect(Kubernetes.RawNoHostSubnetNodes).To(Equal(""))
 			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
-				{mustParseCIDR("10.128.0.0/14"), 23},
+				{ovntest.MustParseIPNet("10.128.0.0/14"), 23},
 			}))
+			Expect(IPv4Mode).To(Equal(true))
 			Expect(IPv6Mode).To(Equal(false))
+			Expect(HybridOverlay.Enabled).To(Equal(false))
 
 			for _, a := range []OvnAuthConfig{OvnNorth, OvnSouth} {
 				Expect(a.Scheme).To(Equal(OvnDBSchemeUnix))
@@ -480,9 +471,9 @@ var _ = Describe("Config Operations", func() {
 			Expect(Kubernetes.CACert).To(Equal(kubeCAFile))
 			Expect(Kubernetes.Token).To(Equal("TG9yZW0gaXBzdW0gZ"))
 			Expect(Kubernetes.APIServer).To(Equal("https://1.2.3.4:6443"))
-			Expect(Kubernetes.ServiceCIDR).To(Equal("172.18.0.0/24"))
+			Expect(Kubernetes.RawServiceCIDRs).To(Equal("172.18.0.0/24"))
 			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
-				{mustParseCIDR("10.129.0.0/14"), 23},
+				{ovntest.MustParseIPNet("10.132.0.0/14"), 23},
 			}))
 
 			Expect(OvnNorth.Scheme).To(Equal(OvnDBSchemeSSL))
@@ -503,6 +494,11 @@ var _ = Describe("Config Operations", func() {
 			Expect(Gateway.VLANID).To(Equal(uint(10)))
 			Expect(Gateway.NodeportEnable).To(BeFalse())
 
+			Expect(HybridOverlay.Enabled).To(BeTrue())
+			Expect(HybridOverlay.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
+				{ovntest.MustParseIPNet("11.132.0.0/14"), 23},
+			}))
+
 			return nil
 		}
 		err = app.Run([]string{app.Name, "-config-file=" + cfgFile.Name()})
@@ -537,10 +533,10 @@ var _ = Describe("Config Operations", func() {
 			Expect(Kubernetes.CACert).To(Equal(kubeCAFile))
 			Expect(Kubernetes.Token).To(Equal("asdfasdfasdfasfd"))
 			Expect(Kubernetes.APIServer).To(Equal("https://4.4.3.2:8080"))
-			Expect(Kubernetes.ServiceCIDR).To(Equal("172.15.0.0/24"))
+			Expect(Kubernetes.RawServiceCIDRs).To(Equal("172.15.0.0/24"))
 			Expect(Kubernetes.RawNoHostSubnetNodes).To(Equal("test=pass"))
 			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
-				{mustParseCIDR("10.130.0.0/15"), 24},
+				{ovntest.MustParseIPNet("10.130.0.0/15"), 24},
 			}))
 
 			Expect(OvnNorth.Scheme).To(Equal(OvnDBSchemeSSL))
@@ -557,6 +553,11 @@ var _ = Describe("Config Operations", func() {
 
 			Expect(Gateway.Mode).To(Equal(GatewayModeLocal))
 			Expect(Gateway.NodeportEnable).To(BeTrue())
+
+			Expect(HybridOverlay.Enabled).To(BeTrue())
+			Expect(HybridOverlay.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
+				{ovntest.MustParseIPNet("11.132.0.0/14"), 23},
+			}))
 			return nil
 		}
 		cliArgs := []string{
@@ -573,18 +574,20 @@ var _ = Describe("Config Operations", func() {
 			"-k8s-apiserver=https://4.4.3.2:8080",
 			"-k8s-cacert=" + kubeCAFile,
 			"-k8s-token=asdfasdfasdfasfd",
-			"-k8s-service-cidr=172.15.0.0/24",
-			"-nb-address=ssl://6.5.4.3:6651",
+			"-k8s-service-cidrs=172.15.0.0/24",
+			"-nb-address=ssl:6.5.4.3:6651",
 			"-no-hostsubnet-nodes=test=pass",
 			"-nb-client-privkey=/client/privkey",
 			"-nb-client-cert=/client/cert",
 			"-nb-client-cacert=/client/cacert",
-			"-sb-address=ssl://6.5.4.1:6652",
+			"-sb-address=ssl:6.5.4.1:6652",
 			"-sb-client-privkey=/client/privkey2",
 			"-sb-client-cert=/client/cert2",
 			"-sb-client-cacert=/client/cacert2",
 			"-gateway-mode=local",
 			"-nodeport",
+			"-enable-hybrid-overlay",
+			"-hybrid-overlay-cluster-subnets=11.132.0.0/14/23",
 		}
 		err = app.Run(cliArgs)
 		Expect(err).NotTo(HaveOccurred())
@@ -592,7 +595,7 @@ var _ = Describe("Config Operations", func() {
 
 	It("overrides config file and defaults with CLI legacy service-cluster-ip-range option", func() {
 		err := ioutil.WriteFile(cfgFile.Name(), []byte(`[kubernetes]
-service-cidr=172.18.0.0/24
+service-cidrs=172.18.0.0/24
 `), 0644)
 		Expect(err).NotTo(HaveOccurred())
 
@@ -601,7 +604,7 @@ service-cidr=172.18.0.0/24
 			cfgPath, err = InitConfig(ctx, kexec.New(), nil)
 			Expect(err).NotTo(HaveOccurred())
 			Expect(cfgPath).To(Equal(cfgFile.Name()))
-			Expect(Kubernetes.ServiceCIDR).To(Equal("172.15.0.0/24"))
+			Expect(Kubernetes.RawServiceCIDRs).To(Equal("172.15.0.0/24"))
 			return nil
 		}
 		cliArgs := []string{
@@ -613,7 +616,29 @@ service-cidr=172.18.0.0/24
 		Expect(err).NotTo(HaveOccurred())
 	})
 
-	It("returns an error when the k8s-service-cidr is invalid", func() {
+	It("accepts legacy service-cidr config file option", func() {
+		err := ioutil.WriteFile(cfgFile.Name(), []byte(`[kubernetes]
+service-cidr=172.18.0.0/24
+`), 0644)
+		Expect(err).NotTo(HaveOccurred())
+
+		app.Action = func(ctx *cli.Context) error {
+			var cfgPath string
+			cfgPath, err = InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(cfgPath).To(Equal(cfgFile.Name()))
+			Expect(Kubernetes.RawServiceCIDRs).To(Equal("172.18.0.0/24"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-config-file=" + cfgFile.Name(),
+		}
+		err = app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("returns an error when the k8s-service-cidrs is invalid", func() {
 		app.Action = func(ctx *cli.Context) error {
 			_, err := InitConfig(ctx, kexec.New(), nil)
 			Expect(err).To(MatchError("kubernetes service network CIDR \"adsfasdfaf\" invalid: invalid CIDR address: adsfasdfaf"))
@@ -639,8 +664,9 @@ cluster-subnets=172.18.0.0/23
 			Expect(err).NotTo(HaveOccurred())
 			Expect(cfgPath).To(Equal(cfgFile.Name()))
 			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
-				{mustParseCIDR("172.15.0.0/23"), 24},
+				{ovntest.MustParseIPNet("172.15.0.0/23"), 24},
 			}))
+			Expect(IPv4Mode).To(Equal(true))
 			Expect(IPv6Mode).To(Equal(false))
 			return nil
 		}
@@ -667,6 +693,21 @@ cluster-subnets=172.18.0.0/23
 		Expect(err).NotTo(HaveOccurred())
 	})
 
+	It("returns an error when the hybrid overlay cluster-subnets is invalid", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("hybrid overlay cluster subnet invalid: CIDR \"adsfasdfaf\" not properly formatted"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-hybrid-overlay-cluster-subnets=adsfasdfaf",
+			"-enable-hybrid-overlay",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
 	It("overrides config file and defaults with CLI legacy --init-gateways option", func() {
 		err := ioutil.WriteFile(cfgFile.Name(), []byte(`[gateway]
 mode=local
@@ -757,7 +798,7 @@ mode=shared
 			Expect(Kubernetes.Token).To(Equal("asdfasdfasdfasfd"))
 			Expect(Kubernetes.APIServer).To(Equal("https://4.4.3.2:8080"))
 			Expect(Kubernetes.RawNoHostSubnetNodes).To(Equal("label=another-test-label"))
-			Expect(Kubernetes.ServiceCIDR).To(Equal("172.15.0.0/24"))
+			Expect(Kubernetes.RawServiceCIDRs).To(Equal("172.15.0.0/24"))
 
 			Expect(OvnNorth.Scheme).To(Equal(OvnDBSchemeSSL))
 			Expect(OvnNorth.PrivKey).To(Equal("/client/privkey"))
@@ -789,11 +830,11 @@ mode=shared
 			"-k8s-cacert=" + kubeCAFile,
 			"-k8s-token=asdfasdfasdfasfd",
 			"-k8s-service-cidr=172.15.0.0/24",
-			"-nb-address=ssl://6.5.4.3:6651,ssl://6.5.4.4:6651,ssl://6.5.4.5:6651",
+			"-nb-address=ssl:6.5.4.3:6651,ssl:6.5.4.4:6651,ssl:6.5.4.5:6651",
 			"-nb-client-privkey=/client/privkey",
 			"-nb-client-cert=/client/cert",
 			"-nb-client-cacert=/client/cacert",
-			"-sb-address=ssl://6.5.4.1:6652,ssl://6.5.4.2:6652,ssl://6.5.4.3:6652",
+			"-sb-address=ssl:6.5.4.1:6652,ssl:6.5.4.2:6652,ssl:6.5.4.3:6652",
 			"-sb-client-privkey=/client/privkey2",
 			"-sb-client-cert=/client/cert2",
 			"-sb-client-cacert=/client/cacert2",
@@ -822,9 +863,9 @@ mode=shared
 
 			Expect(Default.MTU).To(Equal(1500))
 			Expect(Default.ConntrackZone).To(Equal(64321))
-			Expect(Default.RawClusterSubnets).To(Equal("10.129.0.0/14/23"))
+			Expect(Default.RawClusterSubnets).To(Equal("10.132.0.0/14/23"))
 			Expect(Default.ClusterSubnets).To(Equal([]CIDRNetworkEntry{
-				{mustParseCIDR("10.129.0.0/14"), 23},
+				{ovntest.MustParseIPNet("10.132.0.0/14"), 23},
 			}))
 			Expect(Logging.File).To(Equal("/var/log/ovnkube.log"))
 			Expect(Logging.Level).To(Equal(5))
@@ -833,7 +874,7 @@ mode=shared
 			Expect(Kubernetes.Kubeconfig).To(Equal(kubeconfigFile))
 			Expect(Kubernetes.CACert).To(Equal(kubeCAFile))
 			Expect(Kubernetes.Token).To(Equal("TG9yZW0gaXBzdW0gZ"))
-			Expect(Kubernetes.ServiceCIDR).To(Equal("172.18.0.0/24"))
+			Expect(Kubernetes.RawServiceCIDRs).To(Equal("172.18.0.0/24"))
 
 			return nil
 		}
@@ -848,6 +889,149 @@ mode=shared
 		Expect(err).NotTo(HaveOccurred())
 	})
 
+	It("allows configuring a single-stack IPv6 cluster", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(IPv4Mode).To(Equal(false))
+			Expect(IPv6Mode).To(Equal(true))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=fd01::/48/64",
+			"-k8s-service-cidrs=fd02::/112",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("allows configuring a dual-stack cluster", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(IPv4Mode).To(Equal(true))
+			Expect(IPv6Mode).To(Equal(true))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=10.0.0.0/16/24,fd01::/48/64",
+			"-k8s-service-cidrs=172.30.0.0/16,fd02::/112",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("allows configuring a dual-stack cluster with multiple IPv4 cluster subnet ranges", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(IPv4Mode).To(Equal(true))
+			Expect(IPv6Mode).To(Equal(true))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=10.0.0.0/16/24,10.2.0.0/16/24,fd01::/48/64",
+			"-k8s-service-cidrs=172.30.0.0/16,fd02::/112",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("rejects a cluster with IPv4 pods and IPv6 services", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("illegal network configuration: IPv4 cluster subnet, IPv6 service subnet"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=10.0.0.0/16/24",
+			"-k8s-service-cidrs=fd02::/112",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("rejects a cluster with IPv6 pods and IPv4 services", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("illegal network configuration: IPv6 cluster subnet, IPv4 service subnet"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=fd01::/48/64",
+			"-k8s-service-cidrs=172.30.0.0/16",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("rejects a cluster with dual-stack pods and single-stack services", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("illegal network configuration: dual-stack cluster subnet, IPv4 service subnet"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=10.0.0.0/16/24,fd01::/48/64",
+			"-k8s-service-cidrs=172.30.0.0/16",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("rejects a cluster with single-stack pods and dual-stack services", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("illegal network configuration: IPv6 cluster subnet, dual-stack service subnet"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=fd01::/48/64",
+			"-k8s-service-cidrs=172.30.0.0/16,fd02::/112",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("rejects a cluster with multiple single-stack service CIDRs", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("kubernetes service-cidrs must contain either a single CIDR or else an IPv4/IPv6 pair"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=10.0.0.0/16/24",
+			"-k8s-service-cidrs=172.30.0.0/16,172.31.0.0/16",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
+	It("rejects a cluster with dual-stack cluster subnets and single-stack hybrid overlap subnets", func() {
+		app.Action = func(ctx *cli.Context) error {
+			_, err := InitConfig(ctx, kexec.New(), nil)
+			Expect(err).To(MatchError("illegal network configuration: dual-stack cluster subnet, dual-stack service subnet, IPv4 hybrid overlay subnet"))
+			return nil
+		}
+		cliArgs := []string{
+			app.Name,
+			"-cluster-subnets=10.0.0.0/16/24,fd01::/48/64",
+			"-k8s-service-cidrs=172.30.0.0/16,fd02::/112",
+			"-enable-hybrid-overlay",
+			"-hybrid-overlay-cluster-subnets=10.132.0.0/14/23",
+		}
+		err := app.Run(cliArgs)
+		Expect(err).NotTo(HaveOccurred())
+	})
+
 	Describe("OvnDBAuth operations", func() {
 		var certFile, keyFile, caFile string
 
@@ -869,17 +1053,15 @@ mode=shared
 		})
 
 		const (
-			nbURL string = "ssl://1.2.3.4:6641"
-			sbURL string = "ssl://1.2.3.4:6642"
+			nbURL string = "ssl:1.2.3.4:6641"
+			sbURL string = "ssl:1.2.3.4:6642"
 		)
 
 		It("configures client northbound SSL correctly", func() {
-			const nbURLOVN string = "ssl:1.2.3.4:6641"
-
 			fexec := ovntest.NewFakeExec()
 			fexec.AddFakeCmdsNoOutputNoError([]string{
-				"ovn-nbctl --db=" + nbURLOVN + " --timeout=5 --private-key=" + keyFile + " --certificate=" + certFile + " --bootstrap-ca-cert=" + caFile + " list nb_global",
-				"ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-nb=\"" + nbURLOVN + "\"",
+				"ovn-nbctl --db=" + nbURL + " --timeout=5 --private-key=" + keyFile + " --certificate=" + certFile + " --bootstrap-ca-cert=" + caFile + " list nb_global",
+				"ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-nb=\"" + nbURL + "\"",
 			})
 
 			cliConfig := &OvnAuthConfig{
@@ -894,25 +1076,23 @@ mode=shared
 			Expect(a.PrivKey).To(Equal(keyFile))
 			Expect(a.Cert).To(Equal(certFile))
 			Expect(a.CACert).To(Equal(caFile))
-			Expect(a.Address).To(Equal("ssl:1.2.3.4:6641"))
+			Expect(a.Address).To(Equal(nbURL))
 			Expect(a.northbound).To(BeTrue())
 			Expect(a.externalID).To(Equal("ovn-nb"))
 
-			Expect(a.GetURL()).To(Equal(nbURLOVN))
+			Expect(a.GetURL()).To(Equal(nbURL))
 			err = a.SetDBAuth()
 			Expect(err).NotTo(HaveOccurred())
 			Expect(fexec.CalledMatchesExpected()).To(BeTrue(), fexec.ErrorDesc)
 		})
 
 		It("configures client southbound SSL correctly", func() {
-			const sbURLOVN string = "ssl:1.2.3.4:6642"
-
 			fexec := ovntest.NewFakeExec()
 			fexec.AddFakeCmdsNoOutputNoError([]string{
-				"ovn-nbctl --db=" + sbURLOVN + " --timeout=5 --private-key=" + keyFile + " --certificate=" + certFile + " --bootstrap-ca-cert=" + caFile + " list nb_global",
+				"ovn-nbctl --db=" + sbURL + " --timeout=5 --private-key=" + keyFile + " --certificate=" + certFile + " --bootstrap-ca-cert=" + caFile + " list nb_global",
 				"ovs-vsctl --timeout=15 del-ssl",
 				"ovs-vsctl --timeout=15 set-ssl " + keyFile + " " + certFile + " " + caFile,
-				"ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-remote=\"" + sbURLOVN + "\"",
+				"ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-remote=\"" + sbURL + "\"",
 			})
 
 			cliConfig := &OvnAuthConfig{
@@ -927,38 +1107,62 @@ mode=shared
 			Expect(a.PrivKey).To(Equal(keyFile))
 			Expect(a.Cert).To(Equal(certFile))
 			Expect(a.CACert).To(Equal(caFile))
-			Expect(a.Address).To(Equal("ssl:1.2.3.4:6642"))
+			Expect(a.Address).To(Equal(sbURL))
 			Expect(a.northbound).To(BeFalse())
 			Expect(a.externalID).To(Equal("ovn-remote"))
 
-			Expect(a.GetURL()).To(Equal(sbURLOVN))
+			Expect(a.GetURL()).To(Equal(sbURL))
 			err = a.SetDBAuth()
 			Expect(err).NotTo(HaveOccurred())
 			Expect(fexec.CalledMatchesExpected()).To(BeTrue(), fexec.ErrorDesc)
 		})
 
-		It("configures client northbound and southbound Scheme correctly when watch-endpoint specified", func() {
+		const (
+			nbURLLegacy    string = "tcp://1.2.3.4:6641"
+			nbURLConverted string = "tcp:1.2.3.4:6641"
+			sbURLLegacy    string = "tcp://1.2.3.4:6642"
+			sbURLConverted string = "tcp:1.2.3.4:6642"
+		)
 
+		It("configures client northbound TCP legacy address correctly", func() {
 			fexec := ovntest.NewFakeExec()
+			fexec.AddFakeCmdsNoOutputNoError([]string{
+				"ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-nb=\"" + nbURLConverted + "\"",
+			})
 
-			cliConfig := &OvnAuthConfig{
-				Address: "watch-endpoint",
-				PrivKey: keyFile,
-				Cert:    certFile,
-				CACert:  caFile,
-			}
-			MasterHA.ManageDBServers = true
-
-			a, err := buildOvnAuth(fexec, false, cliConfig, &OvnAuthConfig{}, false)
+			cliConfig := &OvnAuthConfig{Address: nbURLLegacy}
+			a, err := buildOvnAuth(fexec, true, cliConfig, &OvnAuthConfig{}, true)
 			Expect(err).NotTo(HaveOccurred())
-			Expect(a.Scheme).To(Equal(OvnDBSchemeSSL))
-			Expect(a.PrivKey).To(Equal(keyFile))
-			Expect(a.Cert).To(Equal(certFile))
-			Expect(a.CACert).To(Equal(caFile))
-			Expect(a.Address).To(Equal("watch-endpoint"))
-			Expect(a.northbound).To(BeFalse())
-			Expect(a.externalID).To(Equal("ovn-remote"))
+			Expect(a.Scheme).To(Equal(OvnDBSchemeTCP))
+			// Config should convert :// to : in addresses
+			Expect(a.Address).To(Equal(nbURLConverted))
+			Expect(a.northbound).To(BeTrue())
+			Expect(a.externalID).To(Equal("ovn-nb"))
 
+			Expect(a.GetURL()).To(Equal(nbURLConverted))
+			err = a.SetDBAuth()
+			Expect(err).NotTo(HaveOccurred())
+			Expect(fexec.CalledMatchesExpected()).To(BeTrue(), fexec.ErrorDesc)
+		})
+
+		It("configures client southbound TCP legacy address correctly", func() {
+			fexec := ovntest.NewFakeExec()
+			fexec.AddFakeCmdsNoOutputNoError([]string{
+				"ovs-vsctl --timeout=15 set Open_vSwitch . external_ids:ovn-nb=\"" + nbURLConverted + "\"",
+			})
+
+			cliConfig := &OvnAuthConfig{Address: nbURLLegacy}
+			a, err := buildOvnAuth(fexec, true, cliConfig, &OvnAuthConfig{}, true)
+			Expect(err).NotTo(HaveOccurred())
+			Expect(a.Scheme).To(Equal(OvnDBSchemeTCP))
+			// Config should convert :// to : in addresses
+			Expect(a.Address).To(Equal(nbURLConverted))
+			Expect(a.northbound).To(BeTrue())
+			Expect(a.externalID).To(Equal("ovn-nb"))
+
+			Expect(a.GetURL()).To(Equal(nbURLConverted))
+			err = a.SetDBAuth()
+			Expect(err).NotTo(HaveOccurred())
 			Expect(fexec.CalledMatchesExpected()).To(BeTrue(), fexec.ErrorDesc)
 		})
 	})
@@ -1052,7 +1256,7 @@ mode=shared
 			generateTests("the scheme is not empty/tcp/ssl",
 				"unknown OVN DB scheme \"blah\"",
 				func() []string {
-					return []string{"address=blah://1.2.3.4:5555"}
+					return []string{"address=blah:1.2.3.4:5555"}
 				})
 
 			generateTests("the address is unix socket and certs are given",
@@ -1069,7 +1273,7 @@ mode=shared
 				"failed to parse OVN DB host/port \"4.3.2.1\": address 4.3.2.1: missing port in address",
 				func() []string {
 					return []string{
-						"address=tcp://4.3.2.1",
+						"address=tcp:4.3.2.1",
 					}
 				})
 
@@ -1077,7 +1281,7 @@ mode=shared
 				"certificate or key given; perhaps you mean to use the 'ssl' scheme?",
 				func() []string {
 					return []string{
-						"address=tcp://1.2.3.4:444",
+						"address=tcp:1.2.3.4:444",
 						"client-privkey=/bar/baz/foo",
 					}
 				})
@@ -1087,7 +1291,7 @@ mode=shared
 			generateTests("the SSL scheme is missing a client CA cert", "",
 				func() []string {
 					return []string{
-						"address=ssl://1.2.3.4:444",
+						"address=ssl:1.2.3.4:444",
 						"client-privkey=" + keyFile,
 						"client-cert=" + certFile,
 						"client-cacert=/foo/bar/baz",
@@ -1097,7 +1301,7 @@ mode=shared
 			generateTests("the SSL scheme is missing a private key file", "",
 				func() []string {
 					return []string{
-						"address=ssl://1.2.3.4:444",
+						"address=ssl:1.2.3.4:444",
 						"client-privkey=/foo/bar/baz",
 						"client-cert=" + certFile,
 						"client-cacert=" + caFile,
@@ -1107,13 +1311,12 @@ mode=shared
 			generateTests("the SSL scheme is missing a client cert file", "",
 				func() []string {
 					return []string{
-						"address=ssl://1.2.3.4:444",
+						"address=ssl:1.2.3.4:444",
 						"client-privkey=" + keyFile,
 						"client-cert=/foo/bar/baz",
 						"client-cacert=" + caFile,
 					}
 				})
-
 		})
 	})
 })
